﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace ConnectToAPI
{
    public partial class frmAPI : Form
    {
        #region Window & Button events
        public frmAPI()
        {
            InitializeComponent();
        }

        private void btnPost_Click(object sender, EventArgs e)
        {
            ClearGridView();
            AssembleXMLRequest();
            PostXML();
            AnalyseXMLResponse();
        }

        private void btnAddCondition_Click(object sender, EventArgs e)
        {
            lstConditions.Items.Add("<condition field=\"" + txtConditionField.Text + "\" type=\"" + cmbOperators.Text + "\">" + txtConditionValue.Text + "</condition>");
            txtConditionField.SelectAll();
            txtConditionField.Focus();
        }

        private void btnDeleteCondition_Click(object sender, EventArgs e)
        {
            if (lstConditions.SelectedIndex > -1)
            {
                lstConditions.Items.RemoveAt(lstConditions.SelectedIndex);
            }
        }

        private void btnClearConditions_Click(object sender, EventArgs e)
        {
            lstConditions.Items.Clear();
        }

        private void btnDeleteField_Click(object sender, EventArgs e)
        {
            if (lstFields.SelectedIndex > -1)
            {
                lstFields.Items.RemoveAt(lstFields.SelectedIndex);
            }
        }

        private void btnClearFields_Click(object sender, EventArgs e)
        {
            lstFields.Items.Clear();
        }

        private void btnAddField_Click(object sender, EventArgs e)
        {
            lstFields.Items.Add("<field>" + txtField.Text.ToUpper() + "</field>");
            txtField.SelectAll();
            txtField.Focus();
        }

        private void ClearGridView()
        {
            dgvResults.SuspendLayout();
            dgvResults.Rows.Clear();
            dgvResults.Columns.Clear();
            dgvResults.ResumeLayout();
        }

        private void btnPostXml_Click(object sender, EventArgs e)
        {
            PostXML();
        }
        #endregion

        #region XML Manipulation & HTTP Posting
        private void AnalyseXMLResponse()
        {
            int Start = Environment.TickCount;

            XmlDocument xmlDoc = new XmlDocument();
            try
            {
                xmlDoc.LoadXml(txtResponse.Text);
            }
            catch
            {
                //Response was not XML, display result in messagebox
                MessageBox.Show(txtResponse.Text);
            }

            //Loop for each <response> element in XML document (should just be one)
            foreach (XmlNode responseNode in xmlDoc.GetElementsByTagName("response"))
            {
                //Loop through the child nodes
                foreach (XmlNode node in responseNode.ChildNodes)
                {
                    if (node.Name == "error")
                    {
                        //The response was an error message
                        MessageBox.Show(node.InnerText);
                        break;
                    }
                    else if (node.Name == "table")
                    {
                        //The response was table data
                        dgvResults.SuspendLayout();
                        XmlNode recordListNode = node.ChildNodes[0];
                        foreach (XmlNode recordNode in recordListNode.ChildNodes)
                        {
                            //Setup columns in DataGridView if this is the first record
                            if (dgvResults.Rows.Count == 0)
                            {
                                //DataGridViewColumn[] columns = new DataGridViewColumn[recordNode.ChildNodes.Count];
                                dgvResults.Columns.Add("ITEM #", "ITEM #");
                                foreach (XmlNode fieldNode in recordNode.ChildNodes)
                                {
                                    DataGridViewColumn column = new DataGridViewColumn();
                                    column.CellTemplate = new DataGridViewTextBoxCell();
                                    column.FillWeight = 1;
                                    column.HeaderText = fieldNode.Attributes["name"].Value;
                                    column.Name = fieldNode.Attributes["name"].Value;
                                    dgvResults.Columns.Add(column);
                                }
                            }

                            //Add new record
                            int i = dgvResults.Rows.Add(new DataGridViewRow());
                            dgvResults.Rows[i].Cells["ITEM #"].Value = recordNode.Attributes["item"].Value;
                            foreach (XmlNode fieldNode in recordNode.ChildNodes)
                            {
                                dgvResults.Rows[i].Cells[fieldNode.Attributes["name"].Value].Value = fieldNode.InnerText;
                            }
                        }
                        dgvResults.ResumeLayout();
                    }
                }
                int Elapsed = (Environment.TickCount) - Start;
                lblTimeAnalysis.Text = "Analysis: " + Elapsed.ToString() + "ms";
            }
        }

        private void AssembleXMLRequest()
        {
            int Start = Environment.TickCount;

            //XML Header
            txtPost.Text = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>" + System.Environment.NewLine;

            //Client Key
            txtPost.Text += "<request clientKey=\"" + txtClientKey.Text + "\">" + System.Environment.NewLine;

            //Table Name & Sort By
            txtPost.Text += "  <table name=\"" + txtTable.Text + "\" sortBy=\"" + txtSortBy.Text + "\" maxRecords=\"" + nudMaxRecords.Value.ToString() + "\">" + System.Environment.NewLine;

            //Add each field
            txtPost.Text += "    <fields>" + System.Environment.NewLine; 
            foreach (object i in lstFields.Items)
            {
                txtPost.Text += "      " + i + System.Environment.NewLine;
            }
            txtPost.Text += "    </fields>" + System.Environment.NewLine;

            //Add each condition
            txtPost.Text += "    <conditions>" + System.Environment.NewLine;
            foreach (object i in lstConditions.Items)
            {
                txtPost.Text += "      " + i + System.Environment.NewLine;
            }

            txtPost.Text += "    </conditions>" + System.Environment.NewLine +
                           "  </table>" + System.Environment.NewLine +
                           "</request>";

            int Elapsed = (Environment.TickCount) - Start;
            lblTimeAssembly.Text = "Assembly: " + Elapsed.ToString() + "ms";
        }

        private void PostXML()
        {
            int Start = Environment.TickCount;
            String postData;

            System.Net.WebRequest request = System.Net.WebRequest.Create(txtURL.Text);
            System.IO.Stream requestStream;
            System.Net.WebResponse response;
            System.IO.StreamReader responseStreamReader;

            //Set ContentType to text/xml because we are sending the XML document as the post content
            request.ContentType = "text/xml";
            request.Method = "POST";

            //Get the XML document as a string
            postData = txtPost.Text;

            //Convert the string and post to the API
            byte[] bytes = Encoding.UTF8.GetBytes(postData);
            request.ContentLength = bytes.Length;
            requestStream = request.GetRequestStream();
            requestStream.Write(bytes, 0, bytes.Length);

            //Get the response
            response = request.GetResponse();
            responseStreamReader = new System.IO.StreamReader(response.GetResponseStream());

            //Populate the response to the screen
            txtResponse.Text = responseStreamReader.ReadToEnd().Trim();

            int Elapsed = (Environment.TickCount) - Start;
            lblTimePost.Text = "Post: " + Elapsed.ToString() + "ms";
        }
#endregion
    }
}
